<?php
/**
 * NAVT Word Press Plugin
 * Copyright (c) 2006-2007 Greg A. Bellucci/Atalaya Studio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * @package NAVT Word Press Plugin
 * @author Greg A. Bellucci <greg[AT]gbellucci[DOT]us
 * @copyright Copyright &copy; 2006-2007 Greg A. Bellucci
 * @license http://www.opensource.org/licenses/mit-license.php The MIT License
 */
require (dirname(__FILE__).'/../../../../wp-config.php');

$initiate_backup = false;

if( isset($_POST['NAVT_xmldata']) ) {

    // change to the configuration
    $xmldata = $_REQUEST['NAVT_xmldata'];
    if(NAVTdbg()) navt_write_debug(NAVT_XML, sprintf("navtajax xml = %s\n", $xmldata));
    if( !empty($xmldata) ) {

        // decode it and strip slashes
        $xml = stripslashes($xmldata);
        if(NAVTdbg()) navt_write_debug(NAVT_XML, sprintf("stripped navtajax xml = %s\n", $xml));

        // Parse the data
        $m_new = parseXMLmap($xml);

        // verify it
        if( is_array($m_new) && count($m_new) > 0) {
            $m_old = get_option(SERIALIZED_MAP);
            // compare the old map with this one
            $is_diff = 1; // compare_maps($m_new, $m_old);
            if($is_diff) {
                if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf(">> Map was changed\n"));
                // update the option
                update_option(SERIALIZED_MAP, $m_new);
                if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf(">> Map updated\n"));
                $status = XML_SUCCESS;
            }// end if
            else {
                // no differences - not updated
                if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf(">> Map had no changes\n"));
                $status = CONFIG_NOT_UPDATED;
            }
            if(NAVTdbg()) navt_write_debug(NAVT_XML, sprintf("Map:\n"), $m_new);
        }
        else {
            // something wrong with the map
            if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf(">> Bad XML data\n"));
            $status = BAD_XML;
        }
    }
    else {
        // no xml data sent
        if(NAVTdbg()) navt_write_debug(NAVT_MAP, sprintf(">> XML data missing\n"));
        $status = XML_MISSING;
    }
}
else if( isset($_REQUEST['NAVT_vercheck']) ) {

    /* Change to the version check checkbox */
    $state = $_POST['NAVT_vercheck'];
    update_option(VERCHECK, $state);
    $status = XML_SUCCESS;
}
else if( isset($_REQUEST['NAVT_default_group']) ) {

    // change to default group for 'on-the-fly' category additions
    $group_name = $_POST['NAVT_default_group'];
    update_option(DEF_ADD_GROUP, $group_name);
    $status = XML_SUCCESS;
}
else if( isset($_REQUEST['NAVT_tips']) ) {

    // change to default group for displaying tips/help
    $tips = $_POST['NAVT_tips'];
    update_option(TIPS, $tips);
    $status = XML_SUCCESS;
}
else if( isset($_REQUEST['NAVT_backup_plugin']) ) {
    $initiate_backup = true;
}

else {
    $status = UNHANDLED_REQUEST;
}

// send back a response
if(NAVTdbg()) navt_write_debug(NAVT_XML, sprintf("xml response: %s\n", $status));

header('Content-Type: text/xml; charset: UTF-8');
header('Cache-Control: must-revalidate');
$expire_offset = 0;
header('Expires: ' . gmdate('D, d M Y H:i:s', time() + $expire_offset) . ' GMT');
print "<?xml version='1.0' encoding='utf-8' ?><status>".$status."</status>";

// Start a backup ?
if( $initiate_backup == true ) {
    do_backup();
}

/**
 * Parse the xml map created by the JavaScript
 * @param string $xml_map
 * @since .95.1
 * @version .95.42 - rewrite
 *
 * returns an array that contains the map
 * some of the debug is commented out here - uncomment it only if you *must* see
 * the xml processing
 */
function parseXMLmap($xml_map)
{
    $group_open = 0;
    $navt_map_open = 0;
    $parser_error = 0;
    $grp = $typ = $idn = $ttl = $nme =
    $opt = $ext = $vals = $lvl = $index = null;
    $m = array();

    if(NAVTdbg()) navt_write_debug(NAVT_XML, sprintf("%s\n", __FUNCTION__));

    // Escape ampersands that aren't part of entities.
    $xml = preg_replace('/&(?!\w{2,6};)/', '&amp;', $xml_map);
    $p = xml_parser_create('UTF-8');
    $result = xml_parse_into_struct($p, $xml, $vals, $index);
    if( !$result ) {
        $parser_error = 1;
        if(NAVTdbg()) navt_write_debug(NAVT_XML,
        sprintf("XML parse error: %s\n", xml_error_string(xml_get_error_code($p))));
    }
    xml_parser_free($p);

    if(NAVTdbg()) navt_write_debug(NAVT_XML, sprintf("Values:\n"), $vals);

    if( !$parser_error ) {

        $in_group = $in_item = $in_map = 0;
        $curItem = array();

        foreach ($vals as $k => $v) {

            if( $v['tag'] == 'NAVTMAP' ) {
                $in_map = ( $v['type'] == 'open' ) ? 1: 0;
                continue;
            }

            if( $in_map ) {

                if( $v[tag] == 'NAVGROUP' ) {

                    if( $v['type'] == 'open' ) {
                        // Starting a new group
                        $in_group = 1;
                        $curItem[GRP] = trim($v['attributes']['NAME']);
                        $curItem[GRP_OPTIONS] = intval(trim($v['attributes']['OPTIONS']));
                        if(NAVTdbg()) navt_write_debug(NAVT_XML,
                        sprintf("\tnew group: %s, global options: %s\n", $curItem[GRP], $curItem[GRP_OPTIONS]));
                    }
                    elseif( $v['type'] == 'close' ) {
                        // completed group
                        $in_group = 0;
                        unset($curItem);
                        $curItem = array();
                        if(NAVTdbg()) navt_write_debug(NAVT_XML, sprintf("\tgroup completed\n"));
                    }

                    continue;
                }

                if( $in_group ) {

                    if( $v['tag'] == 'ITEM' ) {
                        if( $v['type'] == 'open' ) {
                            $in_item = 1;
                            if(NAVTdbg()) navt_write_debug(NAVT_XML, sprintf("\tItem begin\n"));
                        }
                        else {
                            // completed item
                            $in_item = 0;

                            $args = array('grp' => $curItem[GRP], 'typ' => $curItem['TYP'],
                            'idn' => $curItem['IDN'],  'ttl' => $curItem['TTL'], 'nme' => $curItem['NME'],
                            'opt' => $curItem['OPT'] | $curItem[GRP_OPTIONS],  'ext' => $curItem['EXT'],
                            'lvl' => $curItem['LVL'], 'ex2' => $curItem['EX2']);
                            $m[] = navt_format_menu_item($args);
                            if(NAVTdbg()) navt_write_debug(NAVT_XML, sprintf("\titem complete:\n"), navt_format_menu_item($args));
                        }
                        continue;
                    }// end ITEM

                    if( $in_item ) {
                        // creating an item
                        if( ($v['tag'] == 'TYP' || $v['tag'] == 'IDN' ||
                        $v['tag'] == 'TTL' || $v['tag'] == 'NME' ||
                        $v['tag'] == 'OPT' || $v['tag'] == 'EXT' ||
                        $v['tag'] == 'EX2' || $v['tag'] == 'LVL' ) && $v['type'] == 'complete') {

                            // save the value
                            $t = $v['value'];

                            // decode strings
                            if( $v['tag'] == 'TTL' || $v['tag'] == 'NME' || $v['tag'] == 'EXT' || $v['tag'] == 'EX2' ) {
                                $t = str_replace('#amp;', '&', $t);
                            }

                            $curItem[$v['tag']] = $t;
                            if(NAVTdbg()) navt_write_debug(NAVT_XML,
                            sprintf("\t\titem param: %s, value = %s\n", $v['tag'], $t));
                            continue;
                        }

                    }// in item
                }// in group
            }// in map
        }// end for

        if( $in_map || $in_group || $in_item ) {
            // incomplete
            unset($m);
        }
    }

    return($m);
}

/**
 * Compare two navt maps
 *
 * @param array $map1 - new map
 * @param array $map2 - old map
 * @return boolean 1 = changed, 0 = same
 */
function compare_maps($map1, $map2) {

    if(NAVTdbg()) navt_write_debug(NAVT_MAP,
    sprintf("%s map1 size: %s, map2 size: %s\n", __FUNCTION__, count($map1), count($map2)));

    $r = (count($map1) != count($map2)) ? 1: 0; // same size?

    // This loop will end if $r ever becomes something other than zero
    // before we reach the end
    for($i = 0; !$r && $i < count($map1); $i++) {

        $r += ( empty( $map1[$i][GRP] ) || empty( $map1[$i][TYP] ) || empty( $map1[$i][IDN] ) ||
        empty( $map1[$i][NME] ) || empty( $map1[$i][OPT] ) || empty( $map1[$i][EXT] ) ||
        empty( $map1[$i][LVL] ) || empty( $map1[$i][EX2] )) ?
        1: 0;

        $r += ($map1[$i][GRP] == $map2[$i][GRP]) ? 0: 1;
        $r += ($map1[$i][TYP] == $map2[$i][TYP]) ? 0: 1;
        $r += ($map1[$i][IDN] == $map2[$i][IDN]) ? 0: 1;
        $r += ($map1[$i][TTL] == $map2[$i][TTL]) ? 0: 1;
        $r += ($map1[$i][NME] == $map2[$i][NME]) ? 0: 1;
        $r += ($map1[$i][OPT] == $map2[$i][OPT]) ? 0: 1;
        $r += ($map1[$i][EXT] == $map2[$i][EXT]) ? 0: 1;
        $r += ($map1[$i][LVL] == $map2[$i][LVL]) ? 0: 1;
        $r += ($map1[$i][EX2] == $map2[$i][EX2]) ? 0: 1;
    }
    return(($r != 0) ? 1: 0);
}
?>