<?php
/*
Plugin Name: NAVT Word Press Plugin
Plugin URI: http://www.gbellucci.us
Description: menumap include file
Author: Greg A. Bellucci
Author URI: http://www.gbellucci.us
*/

/**
 * NAVT Word Press Plugin
 * Copyright (c) 2006-2007 Greg A. Bellucci/Atalaya Studio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * @package NAVT Word Press Plugin
 * @author Greg A. Bellucci <greg[AT]gbellucci[DOT]us
 * @copyright Copyright &copy; 2006-2007 Greg A. Bellucci
 * @license http://www.opensource.org/licenses/mit-license.php The MIT License
 */
if( !defined('navt_include') ) {
    define('navt_include', 1);
    $dbgfilter = 0;

    /**
     * Bit constants
     *
     */
    define( "BIT00", 0 );
    define( "BIT01", 1 );
    define( "BIT02", 2 );
    define( "BIT03", 4 );
    define( "BIT04", 8 );
    define( "BIT05", 16 );
    define( "BIT06", 32 );
    define( "BIT07", 64 );
    define( "BIT08", 128 );
    define( "BIT09", 256 );
    define( "BIT10", 512 );
    define( "BIT11", 1024 );
    define( "BIT12", 2048 );
    define( "BIT13", 4096 );
    define( "BIT14", 8192 );
    define( "BIT15", 16384 );

    @include('navt_debug.php');    // not shipped with plugin
    if( !function_exists('navt_write_debug') ) { function navt_write_debug($a=0, $b, $c=NULL) {} }

    /**
 * Constants
 */
    define('MAJVER', '95');
    define('MINVER', '462');
    define('SCRIPTVERS',  '.'.MAJVER.'.'.MINVER);  // version applies to all files
    define('SCRIPTNAME',  'NAVT');    // name of this script

    // WordPress and WordPress Minimum platform releases supported
    // @since 95.38

    define('DESCRIPTION', 'Navigation Tool for WordPress');
    define('SUBMENUTITLE','Navigation Tool for WordPress');
    define('PLUGIN_HOMEPAGE', "http://wordpress.org/extend/plugins/wordpress-navigation-list-plugin-navt/");
    define('PLUGIN_FORUM', "http://www.gbellucci.us/?page_id=510");
    define('VERSION_CHECK', '<script src="http://www.gbellucci.us/vchk.php?pkg='.SCRIPTNAME.'&ver='.SCRIPTVERS.'" type="text/javascript"></script>');

    // category options
    define('SHOW_IN_LIST',      BIT01); // category: show category in category list
    define('APPEND_POST_COUNT', BIT02); // category: append post count to category name
    define('SHOW_IF_EMPTY',     BIT03); // category: display even when empty
    define('USE_CAT_DESC',      BIT04); // category: use category description for link title text

    // login options
    define('USE_FORM',          BIT01); // login - create/use login form
    define('REDIRECT_REFER',    BIT02); // login - redirect to referring page
    define('REDIRECT_URL',      BIT03); // login - redirect to specified URL

    // separator options
    define('EMPTY_SPACE_OPTION',BIT00); // separator: empty space
    define('HRULE_OPTION',      BIT01); // separator: hrule
    define('PLAIN_TEXT_OPTION', BIT02); // separator: plaintext
    define('DIVMASK',          (EMPTY_SPACE_OPTION | HRULE_OPTION | PLAIN_TEXT_OPTION));
    define('OPTMASK',           0xFF);

    // page options
    define('ISDRAFTPAGE',       BIT01); // page: has draft status

    // elink options
    //
    define('RELFOLLOW',         BIT01); // elink: link follow option
    define('OPEN_SAMEWIN',      BIT02); // elink: the target URL will open in the same window

    // Author options
    define('SHOW_AVATAR',       BIT01);
    // uses: APPEND_POST_COUNT  BIT02
    // uses: SHOW_IF_EMPTY      BIT03
    define('USE_DEF_AVATAR',    BIT04); // use the default avatar
    define('INC_WEBSITE',       BIT07); // include the user's website address
    define('INC_BIO',           BIT09); // include the user's bio
    define('INC_EMAIL',         BIT11); // include the user's email address

    // -- shared bits -- //
    define('ISPRIVATE',         BIT05); // private - user's must be logged in to see
    define('HAS_DD_OPTION',     BIT06); // item is part of a dropdown list

    define('SETGRAPHICLINK',    BIT07); // Link is a graphic link
    define('NO_LINK_TEXT',      BIT08); // Link text is not displayed
    define('USE_SIDEGRAPHIC',   BIT09); // Link text displayed after graphic
    define('DO_NOT_DISPLAY',    BIT10); // the navigation item is hidden from the screen
    define('HAS_NOSTYLE',       BIT12); // Do not apply class information (navt_list)
    define('NOFOLLOW',          BIT13); // rel=nofollow for everything other than ELINKS

    /** Link style options **/
    define('STANDARD_LINK',          '0');
    define('TEXT_OVER_GRAPHIC',      SETGRAPHICLINK);
    define('TEXT_WITH_SIDE_GRAPHIC', USE_SIDEGRAPHIC);
    define('GRAPHIC_LINK',           NO_LINK_TEXT);

    /**
 * Options token names
 */
    define('OPTPREFIX'     , 'wpnavt_');
    define('SERIALIZED_MAP', OPTPREFIX.'map');
    define('LASTMODIFIED'  , OPTPREFIX.'last_modified');
    define('SCHEME'        , OPTPREFIX.'scheme');
    define('INSTALLED'     , OPTPREFIX.'installed');
    define('ACTIVATED'     , OPTPREFIX.'active');
    define('VERSIONID'     , OPTPREFIX.'version');
    define('VERCHECK'      , OPTPREFIX.'verchk');
    define('DEF_ADD_GROUP' , OPTPREFIX.'addgrp');
    define('TIPS'          , OPTPREFIX.'tips');

    /**
 * Associative array indicies
 */
    define('GRP', 'grp'); // item belongs to this group
    define('TYP', 'typ'); // item type
    define('IDN', 'idn'); // database id
    define('TTL', 'ttl'); // title or url
    define('NME', 'nme'); // alias name
    define('OPT', 'opt'); // item options
    define('EXT', 'ext'); // item extra options
    define('LVL', 'lvl'); // item's hierarchy level
    define('EX2', 'ex2'); // extended options


    /**
 * Navigation Item Types
 */
    define('TYPE_PAGE',   '1'); // menu item is a "page"
    define('TYPE_CAT' ,   '2'); // menu item is a "category"
    define('TYPE_LINK',   '3'); // menu item is an "link" (internal)
    define('TYPE_SEP',    '4'); // menu item is a "separator"
    define('TYPE_ELINK',  '5'); // menu item is a "link" (external)
    define('TYPE_AUTHOR', '6'); // menu item is an author */

    // Carries the XML in the form: JS->PHP
    define('MAPVAR', 'mapdef');

    /**
 * Misc
 */
    define('EMPTYSTR',   '');	   // Empty or blank string constant
    define('HOMEIDN',    0); 	   // Identifier for the 'home' menu item
    define('HOMETXT',   'Home');   // Home Menu Title
    define('HOMESLUG',  'home');   // Home page slug
    define('LOGINIDN',   10000);   // Identifier for the 'login/register' link
    define('SEPIDN',     20000);   // Identifier for dividers
    define('LOGINTXT',  'Login');  // Login text
    define('LOGINSLUG', 'login');  // login slug

    /**
 * Form actions
 */
    define('NOACTION',   0);            // form action default
    define('UPDATEMENU', 2); 	        // form action
    define('RESETMENU',  4);  	        // form action
    define('UNINSTALL',  6);	        // form action
    define('VERSION_CHECKING', 8);      // form action
    define('SET_DEFAULT_NAVGROUP', 10); // form action
    define('CONVERT', 12);

    /**
 * Javascript/PHP constants for classes and control names
 * These suffixes and class names identify specific entities
 * in the DOM.
 */
    define('ID_DEFAULT_GROUP', 'UNASSIGNED'); // Default menu group contains all unassigned items
    define('ID_PAGE_WRAPPER', 'mgrps');       // Parent container for all menu groups
    define('ID_GROUP_TEMPLATE', 'xyzzy');     // menu group template name
    define('ID_GROUP_PREFIX', 'nzp-');        // prepended to the groupname to ensure that name selectors are unique
    define('SCHEME_LIMIT', 5);                // Max menu group color schemes
    define('DROPZONES', 'gDropZones');        // java global array for droppables
    define('ID_SCHEME', 'schemez');           // keeps track of the next scheme
    define('ID_SYSTEXTBOX', 'sectionName');
    define('ID_VUSELECT', 'vuselect');
    define('ID_SYSERR', 'syserr');

    define('SEP', '-');
    define('DELIM', '_');
    define('ISFX', 'im');

    define('KW_CLS', 'cls');
    define('KW_ITEM','mi');         // class or id name component
    define('KW_DROPDOWN', 'dd');    // class or id name component
    define('KW_ALIAS', 'alias');    // class or id name component
    define('KW_RENAME', 're');      // class or id name component
    define('KW_DELETE', 'de');      // class or id name component
    define('KW_ADVANCED', 'ad');    // class or id name component
    define('KW_OK', 'ok');          // class or id name component
    define('KW_CAN',  'can');       // class or id name component
    define('KW_CB1',  'cb1');       // class or id name component
    define('KW_CB2',  'cb2');       // class or id name component
    define('KW_CB3',  'cb3');       // class or id name component
    define('KW_CB4',  'cb4');       // class or id name component
    define('KW_CB5',  'cb5');       // class or id name component
    define('KW_CB6',  'cb6');       // class or id name component
    define('KW_CB7',  'cb7');       // class or id name component
    define('KW_CB8',  'cb8');       // class or id name component
    define('KW_CB9',  'cb9');       // class or id name component
    define('KW_CB10', 'cb10');      // class or id name component
    define('KW_CB11', 'cb11');      // class or id name component
    define('KW_CB13', 'cb13');      // class or id name component
    define('KW_PVT', 'pvt');        // class or id name component
    define('KW_OPTION', 'options'); // class or id name component
    define('KW_NOCLASS', 'nc');     // class or id name component

    // buttons
    define('OKBUTTON',  'okbtn');
    define('CANBUTTON', 'cnbtn');
    define('DELBUTTON', 'debtn');

    // DOM Element Classes
    define('CLS_MSG', 'msg');       // Message class for error messages
    define('CLS_MMENU', 'mmenu');   // menu group menu class
    define('CLS_FDIV',  'fdiv');    // wrapper class for menu items within a menu group
    define('CLS_IMENU', 'imenu');   // Item menu class
    define('CLS_IFORM', 'iform');   // item form class
    define('CLS_DCON', 'dcond');    // group display conditions class
    define('CLS_MFORM', 'mform');   // menu group form class
    define('CLS_SEP_ITEM', 'di');
    define('CLS_EXTLINK', 'ex');
    define('CLS_MDELE', 'mdele');                    // delete confirmation class
    define('CLS_MDELE_OK', CLS_MDELE.DELIM.KW_OK);   // delete ok
    define('CLS_MDELE_CAN', CLS_MDELE.DELIM.KW_CAN); // delete cancel
    define('CLS_DCON_OK', CLS_DCON.DELIM.KW_OK);     // delete ok
    define('CLS_DCON_CAN', CLS_DCON.DELIM.KW_CAN);   // delete cancel
    define('CLS_LEVEL', 'lv');
    define('CLS_DELETED', 'deleted');                // deleted navigation item


    // A SFX (suffix) is added to a DOM ID and is usually
    // 'paired' with an element CLASSNAME. The SFX and ID
    // are separated by DELIM.

    define('MSG_SFX', DELIM.CLS_MSG);         // id suffix for messages
    define('ID_SFX_FDIV', DELIM.CLS_FDIV);    // id suffix for folder div that surrounds all menu item entries
    define('ID_SFX_MMENU', DELIM.CLS_MMENU);  // id suffix for the div containing the menu group rename controls
    define('ID_SFX_IMENU', DELIM.CLS_IMENU);  // id suffix for the div containing the menu alias rename controls
    define('ID_SFX_DCON', DELIM.CLS_DCON);    // id suffix for the div containing the menu display conditions controls
    define('ID_SFX_TXTBX', DELIM.'rtbx');     // id suffix for textbox that contains the new section name when renaming sections
    define('ID_SFX_NTXTBX', DELIM.'ntbx');    // id suffix for textbox that contains an anchor name (external link menu tab)
    define('ID_SFX_UTXTBX', DELIM.'utbx');    // id suffix for textbox that contains the side graphic url
    define('ID_SFX_IMGCLS', DELIM.'itbx');    // id suffix for textbox that contains the target URL (external link menu item)
    define('ID_SFX_ATXTBX', DELIM.'atbx');    // id suffix for textbox that contains the Author Avitar URL
    define('ID_SFX_REDIRECT', DELIM.'rdirtbx');  // id suffix for textbox that contains the redirect URL

    define('ID_SFX_TTL', DELIM.TTL);          // id suffix for ttl (page or category database title)
    define('ID_SFX_OPT', DELIM.OPT);          // id suffix for opt (menu item options)
    define('ID_SFX_NME', DELIM.NME);          // id suffix for nme (page or category menu tab alias name)
    define('ID_SFX_EXT', DELIM.EXT);          // id suffix for ext (usually a graphic path)
    define('ID_SFX_LVL', DELIM.LVL);          // id suffix for lvl
    define('ID_SFX_EX2', DELIM.EX2);          // id suffix for ex2
    define('ID_SFX_CB1', DELIM.KW_CB1);       // id suffix for category/page option 1 checkbox/option
    define('ID_SFX_CB2', DELIM.KW_CB2);       // id suffix for category/page option 2 checkbox/option
    define('ID_SFX_CB3', DELIM.KW_CB3);       // id suffix for category/page option 3 checkbox/option
    define('ID_SFX_CB4', DELIM.KW_CB4);       // id suffix for category/page option 4 checkbox/option
    define('ID_SFX_CB5', DELIM.KW_CB5);       // id suffix for category/page option 5 checkbox/option
    define('ID_SFX_CB6', DELIM.KW_CB6);       // id suffix for category/page option 6 checkbox/option
    define('ID_SFX_CB7', DELIM.KW_CB7);       // id suffix for category/page option 7 checkbox/option
    define('ID_SFX_CB8', DELIM.KW_CB8);       // id suffix for category/page option 8 checkbox/option
    define('ID_SFX_CB9', DELIM.KW_CB9);       // id suffix for category/page option 9 checkbox/option
    define('ID_SFX_CB10', DELIM.KW_CB10);     // id suffix for category/page option 10 checkbox/option
    define('ID_SFX_CB11', DELIM.KW_CB11);     // id suffix for category/page option 11 checkbox/option
    define('ID_SFX_CB13', DELIM.KW_CB13);     // id suffix for category/page option 11 checkbox/option
    define('ID_SFX_ANCHOR_OPTS', DELIM.'ao'); // id suffix for link style options
    define('ID_SFX_OPTION', DELIM.KW_OPTION); // id suffix for the menu item options
    define('ID_SFX_RENAME', DELIM.KW_RENAME); // id suffix for rename control
    define('ID_SFX_DELETE', DELIM.KW_DELETE); // id suffix for delete
    define('ID_SFX_AVATAR_IMG', DELIM.'avt'); // id suffix for avatar graphic
    define('ID_SFX_ADVANCED', DELIM.KW_ADVANCED);
    define('ID_SFX_DD', DELIM.KW_DROPDOWN);   // id suffix for dropdown box menu item
    define('ID_SFX_NC', DELIM.KW_NOCLASS);    // id suffix for no style menu item
    define('ID_SFX_OK', DELIM.KW_OK);         // id suffix for ok class (image)
    define('ID_SFX_CANCEL', DELIM.KW_CAN);    // id suffix for cancel class (image)
    define('ID_SFX_IFORM', DELIM.CLS_IFORM);  // id suffix for item rename form
    define('ID_SFX_MFORM', DELIM.CLS_MFORM);  // id suffix for group rename form
    define('ID_SFX_GROUPNAME', DELIM.'gn');   // id suffix for the menu group name
    define('ID_SFX_MDELE', DELIM.CLS_MDELE);
    define('ID_SFX_SEP', DELIM.CLS_SEP_ITEM);
    define('ID_SFX_EXTLINK', DELIM.CLS_EXTLINK);
    define('ID_SFX_MENUITEM',DELIM.KW_ITEM);
    define('ID_SFX_GROUPOPT', DELIM.'gopt');  // group options
    define('ID_SFX_OPTBOX', DELIM.'optbox');  //  item option box id
    define('ID_SFX_OPNALL', DELIM.'oa');      // expand all items
    define('ID_SFX_CLOALL', DELIM.'ca');      // collapse all items
    define('ID_SFX_GENG_CSS', DELIM.'gcss');  // Generate group css code
    define('ID_SFX_RESTOREALL', DELIM.'resall');
    define('ID_SFX_IKON', DELIM.'ik');        // icon image id
    define('ID_SFX_DC', DELIM.'dc');          // display conditions
    define('ID_SFX_AVATAR_LIST', DELIM.'avtlist');
    define('ID_SFX_URLWINDOWOPT', DELIM.'win');
    define('ID_SFX_URLFOLLOWOPT', DELIM.'fol');

    // More DOM Element Classes
    define('CLS_DROPZONE', 'zone');           // Droppables class
    define('CLS_GROUP_SCHEME', 'cs');         // Menu group color scheme class (prefix)
    define('CLS_MENUGROUP', 'mg');	          // class for menu group containers
    define('CLS_MENUITEM_HANDLE', 'mh');      // class used for each menu item (page/category)
    define('CLS_NODRAG', 'nd');               // Menu group handle (stationary group)
    define('CLS_STATIONARY_GROUP','fx');      // Fixed menu group (no dragging)
    define('CLS_MENUGROUP_HANDLE','gh');      // Class used for dragging menu groups
    define('CLS_GROUP_NAV', 'gnav');          // group navigation container class (group menu)
    define('CLS_MENULINK', DELIM.'ml');       // anchor class for group menu items
    define('CLS_OPTBOX', DELIM.'opbx');       // anchor class for menu alias rename
    define('CLS_PGCT', KW_ITEM);              // general menu item class
    define('CLS_ITEMLINK', DELIM.'il');       // anchor class for item menu items
    define('CLS_DRAG', 'nrdrag');             // unrestricted draggable
    define('CLS_RDRAG', 'rdrag');             // restricted draggable
    define('CLS_ITMPL', 'tplt');              // template item class
    define('CLS_SEP', 'divider');             // Item separator class for styling
    define('CLS_BOX_OPEN', 'box-open');       // Highlight an open alias rename box
    define('CLS_BOX_CLOSE', 'box-closed');    // Highlight an open alias rename box
    define('CLS_MOVED', 'moved');             // class for moved menu items
    define('CLS_CHANGE', 'change');           // class for changed alias name
    define('CLS_DISABLED', 'disabled');       // class for displaying disabled items
    define('CLS_HAS_FOCUS', 'hasfocus');
    define('CLS_NEWGROUP', 'new');            // class for new menu groups
    define('CLS_ADD_GROUP', 'newgroup');      // class for new group (button)
    define('CLS_MSORT_TAG','li');	          // Tag used for menu item draggables
    define('CLS_GSORT_TAG', 'div');           // Tag used for group draggables
    define('CLS_ADD_ITEM', 'newitem');        // class for new item (button)
    define('CLS_PRIVATE', 'private');         // item marked private
    define('CLS_DRAFTPAGE', 'draft');         // item marked as a draftpage
    define('CLS_ERRBOX', 'errbox');           // Borders an input textbox
    define('CLS_GRPMENUITEM', 'gmi');         // Group menu items

    define('ALIAS_ERROR',      'xe0');
    define('PLAINTEXT_ERROR',  'xe1');
    define('ELINK_URL_ERROR',  'xe2');
    define('AVATAR_URL_ERROR', 'xe3');
    define('IMG_CLASS_ERROR',  'xe4');

    define('CLS_PAGE',  DELIM.TYPE_PAGE.ISFX);
    define('CLS_CAT',   DELIM.TYPE_CAT.ISFX);
    define('CLS_USER',  DELIM.TYPE_AUTHOR.ISFX);
    define('CLS_ELINK', DELIM.TYPE_ELINK.ISFX);
    define('CLS_LLINK', DELIM.TYPE_LINK.ISFX);

    define('CLS_ELINK_ICON',  'eicon');
    define('CLS_PAGE_ICON',   'picon');
    define('CLS_CAT_ICON',    'cicon');
    define('CLS_LINK_ICON',   'iicon');
    define('CLS_HOME_ICON',   'hicon');
    define('CLS_ADMIN_ICON',  'aicon');
    define('CLS_DIVID_ICON',  'dicon');
    define('CLS_AUTHOR_ICON', 'wicon');

    define('CLS_MLINK', 'mbtn');
    define('CLS_ILINK', 'ibtn');
    define('CLS_MLINK_OK',  CLS_MLINK.DELIM.KW_OK);     // anchor class for group menu items
    define('CLS_MLINK_CAN', CLS_MLINK.DELIM.KW_CAN);    // anchor class for group menu items
    define('CLS_ILINK_OK',  CLS_ILINK.DELIM.KW_OK);     // anchor class for group menu items
    define('CLS_ILINK_DEL', CLS_ILINK.DELIM.KW_DELETE); // anchor class for group menu items
    define('CLS_ILINK_CAN', CLS_ILINK.DELIM.KW_CAN);    // anchor class for group menu items
    define('CLS_AV_SELECT', 'avselect');
    define('CLS_NOSHOW', 'nsh');
    define('CLS_ANCHOR_SELECT', 'ancselect');

    define('MAX_GROUP_NAME', 25);
    define('MAX_ALIAS_NAME', 27);
    define('MAX_URL_NAME', 80);
    define('MAX_LEN', 80);
    define('MAX_LEVELS', 4);
    define('STY_HIDDEN', 'display: none;');   // style for hidden controls/forms
    define('HELP_PAGE', 'http://gbellucci.us?cat=20');   // relative to plugin directory
    define('HELP_WIDTH', 600);

    //Images with paths relative to the plugin directory
    define('IMG_UNPAGE_ICON',   '../images/ikon_page.gif');
    define('IMG_PAGE_ICON',     '../images/ikon_page.gif');
    define('IMG_DRAFT_ICON',    '../images/ikon_draft.gif');
    define('IMG_CATEGORY_ICON', '../images/ikon_category.gif');
    define('IMG_LINK_ICON',     '../images/ikon_link.gif');
    define('IMG_DIVID_ICON',    '../images/ikon_tab.gif');
    define('IMG_AUTHOR_ICON',    '../images/ikon_user.gif');
    define('IMG_HOME_ICON',     '../images/ikon_web-page.gif');
    define('IMG_ADMIN_ICON',    '../images/ikon_user-login.gif');
    define('IMG_BKGD',          '../images/btn.jpg');
    define('IMG_HDR',           '../images/btn.jpg');
    define('IMG_BLANK',         'images/blank.gif');
    define('IMG_AVATAR',        'images/default_avatar.jpg');
    define('IMG_PLUS',          '../images/ikon_indent.gif');
    define('IMG_MINUS',         '../images/ikon_outdent.gif');
    define('IMG_HELP_ICON',     '../images/ikon_help.gif');


    // images with paths relative to the plugin css directory
    define('IMG_OK',            '../images/ikon_accept.gif');
    define('IMG_CANCEL',        '../images/ikon_cancel.gif');
    define('IMG_DELETE',        '../images/ikon_delete.gif');
    define('IMG_SPINNER',       'images/spinner.gif');
    define('AVATAR_IMAGES',     '/avatars');

    // script locations
    define('INC_NAVT', '/js/navt.js.php');
    define('INC_JQUERY', '/js/jquery.js');
    define('INC_NAVTOPTS', '/js/navtoptions.js.php');

    // script versions
    define('VER_NAVT', SCRIPTVERS);


    // Stray options
    define('ALTBLOGPAGE', 'alt_blog_page'); /* database option */
    define('SHOWONFRONT', 'show_on_front'); /* database option */

    // AJAX Return Codes
    define('XML_SUCCESS', 0);
    define('CONFIG_NOT_UPDATED', 1);
    define('XML_MISSING', 2);
    define('BAD_XML', 3);
    define('UNHANDLED_REQUEST', 4);

    // Backup Restore return status
    define('RESTORE_PARSER_ERROR', 'Restore - XML Parser failed');
    define('RESTORE_NO_FILE_NAME', 'Restore - no file name specified');
    define('RESTORE_COMPLETE', 'Restore Completed');

    // Anchor title text
    define('GROUP_RENAME_TEXT', 'Rename menu group');
    define('GROUP_DELETE_TEXT', 'Remove menu group');
    define('ITEM_RENAME_TEXT',  'Rename alias for');
    define('OK_TEXT', 'Click to proceed');
    define('CANCEL_TEXT', 'Click to cancel and close');
    define('DELETE_TEXT', 'Click to delete this item');
    define('DEF_SEPTEXT', 'Empty Space');
    define('DEF_SUBHEADING', 'Plain Text');
    define('DEF_HRULETEXT', 'Horiz Rule');
    define('DEF_ELINKTEXT', 'External Link');

    // Tab types (CSS CLASSES)
    define('PAGEITEM',     'page_item');
    define('CURPAGEITEM',  'current_'.PAGEITEM);
    define('TAB_CATEGORY', 'categorytab');
    define('TAB_PAGE',     'pagetab');
    define('TAB_ADMIN',    'admintab');
    define('TAB_LINK',     'linktab');
    define('TAB_ELINK',    'elinktab');
    define('TAB_HOME',     'hometab');
    define('TAB_AUTHOR',   'authortab');
    define('TAB_SUBHEAD',  'h2tab');
    define('TAB_HRULE',    'hruletab');
    define('TAB_EMPTY',    'emptytab');

    // Divider Options text
    define('OPT_EMPTYTEXT', 'Empty Space');
    define('OPT_HRULE',     'Horizontal Rule');
    define('OPT_PTEXT',     'Plain Text');

    // Group Menu Text
    define('GM_RENAME', 'Rename Group');
    define('GM_DELETE', 'Delete Group');
    define('GM_ADD_LINK', 'Add External Link');
    define('GM_ADD_DIVIDER', 'Add Divider');
    define('GM_DROPDOWN_FORMAT', 'Create HTML Select');
    define('GM_OPEN_ALL', 'Expand All Items');
    define('GM_CLOSE_ALL', 'Collapse All Items');
    define('GM_DISP_COND', 'Display Conditions');
    define('GM_GENG_CSS', 'Generate Group CSS');
    define('GM_RESTORE_ALL', 'Restore Unused Items');
    define('GM_NOSTYLE', 'Exclude tag styles');

    define('VALL',        '0');
    define('VPAGES',      '1');
    define('VPAGES_CATS', '2');
    define('VCATS',       '3');
    define('VUSERS',      '4');
    define('VLINKS',      '5');
    define('HITEMS',      '6');

    // Restore options
    define('RESTORE_IGNORE', '1');
    define('RESTORE_MAP_ITEMS', '2');
    define('RESTORE_MERGE', '3');
    define('RESTORE_CREATE_MERGE', '4');

    // handy functions
    function q_($v) {printf("'%s'", $v);} // echo quoted string
    function e_($v) {printf("%s", $v);}   // echo unquoted string
    function set_klass($strip_style, $str) {
        $r = (($strip_style != 0) ? '': $str);
        return($r);
    }

}// end if !defined
?>