<?php
/*
Plugin Name: WP Navigation Tool (NAVT)
Plugin URI: http://www.gbellucci.us/?page_id=510
Description: Create, organize and manage your site menus and navigation lists by logically grouping your pages and categories via a drag'n drop interface. Manage your navigation lists from the NAVT Lists menu tab in the <a href="/wp-admin/edit.php">Manage</a> Menu.
Version: .95.46.2
Author: Greg Bellucci
Author URI: http://www.gbellucci.us
*/

@define('NAVT_PLUGINPATH', (DIRECTORY_SEPARATOR != '/') ? str_replace(DIRECTORY_SEPARATOR, '/', dirname(__FILE__)) : dirname(__FILE__));
require('includes/navtinc.php');
require('app/navt.php');


/**
 * NAVT Word Press Plugin
 * Copyright (c) 2006-2007 Greg A. Bellucci/Atalaya Studio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * @package NAVT Word Press Plugin
 * @author Greg A. Bellucci <greg[AT]gbellucci[DOT]us
 * @copyright Copyright (c) 2006-2007 Greg A. Bellucci
 * @license http://www.opensource.org/licenses/mit-license.php The MIT License.
 */

/* ------------------------------------------------------------------------------------------------------------
   navt_getlist - user API
   ------------------------------------------------------------------------------------------------------------

Function call syntax:
---------------------

$sNavGroupName (no default - note: you cannot specify the 'unassigned' menu group)
   (string) The name of the requested navigation group- Note: the name *must* match the group name you defined when the group was created.

$bEcho (default=true)
   (boolean) Instructs the function to echo the html output. If set to false, the output is returned. Default is true.

$sTitle (default='')
   (string) An optional title to appear before the menu. You may specify tags/class (i.e. '<h3 class="cls">Menu Title<h3>' )

$sBefore (default='ul')
   (string) The opening tag(s) that encapsulates the menu (without the beginning or ending angle brackets) Combinations of tags can be used as in:
   'ul><div class="my_menu_class"' (Note that the beginning and ending angle brackets are not included. The default is 'ul'.

$sAfter (default='/ul')
   (string) The closing tag(s) that match the $sBefore tag (without the beginning or ending angle brackets) Use the closing tag combinations for
   multiple tags as in: "/div></ul" for the $sBefore example above. The default is '/ul'.

$sBeforeItem (default='li')
   (string) The opening tag(s) that encapsulates each menu item (without the beginning or ending angle brackets). The default is 'li'.

$sAfterItem (default='/li')
   (string) The closing tag(s) that matches the tags used in $sBeforeItem (without the beginning or ending angle brackets). The default is '/li'.

 return HTML formatted output or NULL if $bEcho=true

*/

function navt_getlist($sNavGroupName, $bEcho=true, $sTitle='', $sBefore='ul', $sAfter='/ul', $sBeforeItem='li', $sAfterItem='/li') {

    $out = NAVT::getlist($sNavGroupName, $sTitle, $sBefore, $sAfter, $sBeforeItem, $sAfterItem);
    if( $bEcho ) {
        echo $out;
        $out = null;
    }
    return($out);
}